# Portal::Accounting::DB::Ledger.pm
# Created by JT Moree, Xperience, Inc.
# Thu Jan  9 13:45:24 UTC 2003

#$Id: Ledger.pm,v 1.12 2003/08/12 21:20:36 pcxuser Exp $
#This code is licensed under the GPL

=head1 NAME

Ledger - Object used to interact with Database dealing with Ledger functions.

=head1 SYNOPSIS

  use Portal::Accounting::DB::Ledger;
  my $obj = Portal::Accounting::DB::Ledger->new;
  if ($obj->didErrorOccur())
  {
    die $obj->errorMessage();
  }

=head1 DESCRIPTION

Ledger contains methods for interchanging ledger data with the database.

=cut
package Portal::Accounting::DB::Ledger;
use strict;
use Portal::Accounting::Objects::Ledger;
use Portal::Accounting::DB::Main;
use vars qw($AUTOLOAD $VERSION @ISA @EXPORT);

require Exporter;

@ISA = qw(Portal::Accounting::DB::Main Exporter AutoLoader);
@EXPORT = qw();

$VERSION = '.2';

=head1 Exported FUNCTIONS

=item scalar new(void)

    Creates a new instance of the Portal::Accounting::DB::Ledger
    object.

=cut
sub new
{
  my $class = shift;
  my $self = $class->SUPER::new(@_);
  my %args = ( @_ );

  if ($self->didErrorOccur)
  {
    $self->prefixError();
    return $self;
  }

  # instantiate anything unique to this module

  # do validation
  if (!$self->isValid)
  {
    # the error is set in the isValid() method.
    return $self;
  }

  # do anything else you might need to do.

  return $self;
}

=item bool isValid(void)

 Returns 0 or 1 to indicate if the object is valid.
 The error will be available via errorMessage().

=cut

sub isValid
{
  my $self = shift;

  # make sure our Parent class is valid.
  if (not $self->SUPER::isValid())
  {
    $self->prefixError();
    return 0;
  }
  # validate our parameters.

  if ($self->numInvalid() > 0 || $self->numMissing() > 0)
  {
    $self->postfixError($self->genErrorString("all"));
    return 0;
  }

  return 1;
}

=item bool updateLedger(ledgerCode, color, businessType, cashOrAccrual)

    Updates a database ledger entry when given valid data.
    Only certain items can be changed on a ledger:
      color, businessType, cashOrAccrual

=cut
sub updateLedger
{
  my $self = shift;
  my %args = ( ledgerCode => "", color => "", businessType => "", cashOrAccrual => "", @_ ) ;
  my $color = "";
  my $businessType = "";
  my $cashOrAccrual = "";
  my $sql = "update ledger_tb set ";

  if (defined $args{ledgerObject})
  {
    if (! $args{ledgerObject}->isValid)
    {
      $self->setError("LedgerObject is invalid.<br>\n".$args{ledgerObject}->errorMessage);
      return 0;
    }
    $color = $args{ledgerObject}->{color};
    $cashOrAccrual = $args{ledgerObject}->{cashOrAccrual};
    $businessType = $args{ledgerObject}->{businessType};
  }
  elsif ($args{ledgerCode} ne "")
  {
    $color = $args{color};
    $cashOrAccrual = $args{cashOrAccrual};
    $businessType = $args{businessType};
  }
  else
  {
    $self->setError("LedgerCode or LedgerObject is required.<br>\n");
    return 0;
  }
  my $ledgerOrig = ($self->getLedgers(ledgerCode => $args{ledgerCode}))[0];
  if ($self->didErrorOccur)
  {
    $self->setError("Error while retrieving original LedgerObject for $args{ledgerCode}.<br>\n");
    return 0;
  }
  if ($color eq $ledgerOrig->{color})
  {  $color = ""; }
  if ($cashOrAccrual eq $ledgerOrig->{cashOrAccrual})
  {  $cashOrAccrual = ""; }
  if ($businessType eq $ledgerOrig->{businessType})
  {  $businessType = ""; }
  #must have 1 of three options
  my $num = 0;
  if ($args{color} ne "")
  {
    $num++;
    $sql .= " color = '$args{color}' ";
  }
  if ($num > 0)
  { $sql .= ", "; }
  if ($args{cashOrAccrual} ne "")
  {
    $num++;
    if ($args{cashOrAccrual} !~ /^(cash|accrual)$/i)
    {
      $self->setError("Invalid CashOrAccrual: '$args{cashOrAccrual}'.<br>\n");
      return 0;
    }
    my $cashOrAccrual = "";
    if ($args{cashOrAccrual} =~ /^cash$/i)
    {  $cashOrAccrual = "true";}
    elsif ($args{cashOrAccrual} =~ /^accrual$/i)
    { $cashOrAccrual = "false";}

    $sql .= " cash_or_accrual = '$cashOrAccrual'";
  }
  if ($num > 0)
  { $sql .= ", "; }
  if ($args{businessType} ne "")
  {
    $num++;
    $sql .= " business_type = '$args{businessType}' ";
  }
  if ($num < 1)
  {
    $self->setError("at least one field is required (color, cashOrAccrual, businessType).<br>\n");
    return 0;
  }
  $sql .= "WHERE code = '$args{ledgerCode}'";
  $self->{dbHandle}->write(sql => $sql);
  if ($self->{dbHandle}->didErrorOccur)
  {
    $self->setError($self->{dbHandle}->errorMessage);
    return 0;
  }

  $self->{dbHandle}->commit;
  if ($self->{dbHandle}->didErrorOccur)
  {
    $self->setError($self->{dbHandle}->errorMessage);
    return 0;
  }

  return 1;
}

=item bool createLedger(ledger => Portal::Accounting::Objects::Ledger)

    Creates a database ledger entry when given a valid Ledger Object.

=cut
sub createLedger
{
  my $self = shift;
  my %args = ( ledger => undef, @_ ) ;
  my $cashOrAccrual = "";
  my $l = $args{ledger};
  my $ledgerName = "";

  if (not defined $args{ledger})
  {
    $self->setError("Error! Ledger not defined.  Cannot create!<br>\n");
    return 0;
  }
  if ($args{ledger}->isValid == 0)
  {
    $self->setError("Error! Ledger is invalid.  Cannot create!<br>\n");
    return 0;
  }
  if ($l->{cashOrAccrual} eq "cash")
  {  $cashOrAccrual = "true"; }
  else
  {  $cashOrAccrual = "false"; }
  #need to calculate start and end Dates for quarters for first year
  #technically this shoudl be done for each known year found in $l but there should only be one for creation
  my $fiscalYear = (sort keys %{$l->{knownYears}})[0];
  my @end = $l->getQuarterEnds($fiscalYear);
  if (not defined @end)
  {
    $self->setError("Could not insert '".$l->{name}."'! Bad Quarter End Calculations!<br>'\n" );
    return 0;
  }
  my @start = $l->getQuarterStarts($fiscalYear);
  if (not defined @start)
  {
    $self->setError("Could not insert '".$l->{name}."'! Bad Quarter Start Calculations!<br>'\n" );
    return 0;
  }
  my ($fyYear, $fyMo, $fyDay) = split ("-",$end[3]);
  my $insert = "insert into ledger_tb (name, code, color, cash_or_accrual, ";
  $insert .= "  fy_mo, fy_day, business_type," ;
  $insert .= "  start1, start2, start3, start4, start5, start6, start7, start8, start9, start10, start11, start12,";
  $insert .= "  offset1, offset2, offset3, offset4, offset5, offset6, offset7, offset8, offset9, offset10, offset11, offset12)";
  $insert .= " values ('$l->{name}', '$l->{code}', '$l->{color}', $cashOrAccrual, ";
  $insert .= "  '$fyMo', '$fyDay', $l->{businessType}, ";
  $insert .= " '".$l->start(1)."',";
  $insert .= " '".$l->start(2)."',";
  $insert .= " '".$l->start(3)."',";
  $insert .= " '".$l->start(4)."',";
  $insert .= " '".$l->start(5)."',";
  $insert .= " '".$l->start(6)."',";
  $insert .= " '".$l->start(7)."',";
  $insert .= " '".$l->start(8)."',";
  $insert .= " '".$l->start(9)."',";
  $insert .= " '".$l->start(10)."',";
  $insert .= " '".$l->start(11)."',";
  $insert .= " '".$l->start(12)."',";
  $insert .= " '".$l->offset(1)."',";
  $insert .= " '".$l->offset(2)."',";
  $insert .= " '".$l->offset(3)."',";
  $insert .= " '".$l->offset(4)."',";
  $insert .= " '".$l->offset(5)."',";
  $insert .= " '".$l->offset(6)."',";
  $insert .= " '".$l->offset(7)."',";
  $insert .= " '".$l->offset(8)."',";
  $insert .= " '".$l->offset(9)."',";
  $insert .= " '".$l->offset(10)."',";
  $insert .= " '".$l->offset(11)."',";
  $insert .= " '".$l->offset(12)."')";
  if ($l->didErrorOccur)
  {
    $self->prefixError($l->errorMessage."\n" );
    $self->prefixError("DEBUG: $insert<br>".$l->start(1)."\n" );
    return 0;
  }
  $self->{dbHandle}->write(sql => $insert);
  if ($self->{dbHandle}->didErrorOccur)
  {
    $self->setError("Could not insert '$l->{name}'<br>\n" . $self->{dbHandle}->errorMessage);
    return 0;
  }
  $insert = "insert into known_years_tb ";
  $insert .= "(ledger_code, closed, year, ";
  $insert .= "start_date, end_date, q1_start, q1_end, q2_start, q2_end, q3_start, q3_end, q4_start, q4_end) ";
  $insert .= "values ('$l->{code}', 'false', '$fiscalYear', ";
  $insert .= "'$start[0]', '$end[3]', ";
  $insert .= "'$start[0]', '$end[0]', ";
  $insert .= "'$start[1]', '$end[1]', ";
  $insert .= "'$start[2]', '$end[2]', ";
  $insert .= "'$start[3]', '$end[3]') ";
  $self->{dbHandle}->write(sql => $insert);
  if ($self->{dbHandle}->didErrorOccur)
  {
    $self->setError($self->{dbHandle}->errorMessage);
    return 0;
  }
  #############################
  #!!!!need to fill in variables_tb and entity_tb for CGI
  my $variables = "insert into variables_tb (working_year, ledger_code) values ('$fiscalYear', '$l->{code}')";
  my $entity = "insert into entity_tb (name, database_name) values ('$l->{name}', '$l->{name}')";
  $self->{dbHandle}->write(sql => $variables);
  if ($self->{dbHandle}->didErrorOccur)
  {
    $self->setError($self->{dbHandle}->errorMessage);
    return 0;
  }
  $self->{dbHandle}->write(sql => $entity);
  if ($self->{dbHandle}->didErrorOccur)
  {
    $self->setError($self->{dbHandle}->errorMessage);
    return 0;
  }
  ########################

  $self->{dbHandle}->commit;
  if ($self->{dbHandle}->didErrorOccur)
  {
    $self->setError($self->{dbHandle}->errorMessage);
    return 0;
  }

  return 1;
}

=item int deleteLedger(ledgerCode)

 Attempts to delete the specified ledger.
 Returns 1 on success, 0 on errors and sets the errorMessage,
 -1 if the ledger doesn't exist anymore.

=cut

sub deleteLedger
{
  my $self = shift;
  my %args = ( ledgerCode => "", @_ );
  my $ledgerCode = $args{ledgerCode};

  # first verify the ledger still exists to delete.
  my @ledger = $self->getLedgers(ledgerCode => $ledgerCode);
  if (@ledger)
  {
    my $ledger = $ledger[0];
    # first we need to remove the ledger name from the entity_tb table.
    $self->{dbHandle}->write(sql => "DELETE FROM entity_tb WHERE name = ?", plug => [ $ledger->{name} ]);
    if ($self->{dbHandle}->didErrorOccur)
    {
      $self->setError($self->{dbHandle}->errorMessage);
      return 0;
    }

    # now we need to remove all entries from the time_tracker_config_tb and tips_tb tables that reference this ledger.
    $self->{dbHandle}->write(sql => "DELETE FROM time_tracker_config_tb WHERE ledger_code = ?", plug => [ $ledgerCode ]);
    if ($self->{dbHandle}->didErrorOccur)
    {
      $self->setError($self->{dbHandle}->errorMessage);
      return 0;
    }

    $self->{dbHandle}->write(sql => "DELETE FROM tips_tb WHERE ledger_code = ?", plug => [ $ledgerCode]);
    if ($self->{dbHandle}->didErrorOccur)
    {
      $self->setError($self->{dbHandle}->errorMessage);
      return 0;
    }

    # ledger_tb
    $self->{dbHandle}->write(sql => "DELETE FROM ledger_tb WHERE code = ?", plug => [ $ledgerCode]);
    if ($self->{dbHandle}->didErrorOccur)
    {
      $self->setError($self->{dbHandle}->errorMessage);
      return 0;
    }

    # variables_tb
    $self->{dbHandle}->write(sql => "DELETE FROM variables_tb WHERE ledger_code = ?", plug => [ $ledgerCode]);
    if ($self->{dbHandle}->didErrorOccur)
    {
      $self->setError($self->{dbHandle}->errorMessage);
      return 0;
    }

    # known_years_tb
    $self->{dbHandle}->write(sql => "DELETE FROM known_years_tb WHERE ledger_code = ?", plug => [ $ledgerCode]);
    if ($self->{dbHandle}->didErrorOccur)
    {
      $self->setError($self->{dbHandle}->errorMessage);
      return 0;
    }

    # sub_entity_tb
    $self->{dbHandle}->write(sql => "DELETE FROM sub_entity_tb WHERE ledger_code = ?", plug => [ $ledgerCode]);
    if ($self->{dbHandle}->didErrorOccur)
    {
      $self->setError($self->{dbHandle}->errorMessage);
      return 0;
    }

    # journal_entry_tb
    $self->{dbHandle}->write(sql => "DELETE FROM journal_entry_tb WHERE ledger_code = ?", plug => [ $ledgerCode]);
    if ($self->{dbHandle}->didErrorOccur)
    {
      $self->setError($self->{dbHandle}->errorMessage);
      return 0;
    }

    # debitcredit_tb
    $self->{dbHandle}->write(sql => "DELETE FROM debitcredit_tb WHERE ledger_code = ?", plug => [ $ledgerCode]);
    if ($self->{dbHandle}->didErrorOccur)
    {
      $self->setError($self->{dbHandle}->errorMessage);
      return 0;
    }

    # invoice_tb
    $self->{dbHandle}->write(sql => "DELETE FROM invoice_tb WHERE ledger_code = ?", plug => [ $ledgerCode]);
    if ($self->{dbHandle}->didErrorOccur)
    {
      $self->setError($self->{dbHandle}->errorMessage);
      return 0;
    }

    # invoice_items_tb
    $self->{dbHandle}->write(sql => "DELETE FROM invoice_items_tb WHERE ledger_code = ?", plug => [ $ledgerCode]);
    if ($self->{dbHandle}->didErrorOccur)
    {
      $self->setError($self->{dbHandle}->errorMessage);
      return 0;
    }

    # billing_tb
    $self->{dbHandle}->write(sql => "DELETE FROM billing_tb WHERE ledger_code = ?", plug => [ $ledgerCode]);
    if ($self->{dbHandle}->didErrorOccur)
    {
      $self->setError($self->{dbHandle}->errorMessage);
      return 0;
    }

    # group_tb
    $self->{dbHandle}->write(sql => "DELETE FROM group_tb WHERE ledger_code = ?", plug => [ $ledgerCode]);
    if ($self->{dbHandle}->didErrorOccur)
    {
      $self->setError($self->{dbHandle}->errorMessage);
      return 0;
    }

    # client_group_tb
    $self->{dbHandle}->write(sql => "DELETE FROM client_group_tb WHERE ledger_code = ?", plug => [ $ledgerCode]);
    if ($self->{dbHandle}->didErrorOccur)
    {
      $self->setError($self->{dbHandle}->errorMessage);
      return 0;
    }

    # asset_tb
    $self->{dbHandle}->write(sql => "DELETE FROM asset_tb WHERE ledger_code = ?", plug => [ $ledgerCode]);
    if ($self->{dbHandle}->didErrorOccur)
    {
      $self->setError($self->{dbHandle}->errorMessage);
      return 0;
    }

    # reminder_tb
    $self->{dbHandle}->write(sql => "DELETE FROM reminder_tb WHERE ledger_code = ?", plug => [ $ledgerCode]);
    if ($self->{dbHandle}->didErrorOccur)
    {
      $self->setError($self->{dbHandle}->errorMessage);
      return 0;
    }

    # custom_report_tb
    $self->{dbHandle}->write(sql => "DELETE FROM custom_report_tb WHERE ledger_code = ?", plug => [ $ledgerCode]);
    if ($self->{dbHandle}->didErrorOccur)
    {
      $self->setError($self->{dbHandle}->errorMessage);
      return 0;
    }

    # acct_bal_tb
    $self->{dbHandle}->write(sql => "DELETE FROM acct_bal_tb WHERE ledger_code = ?", plug => [ $ledgerCode]);
    if ($self->{dbHandle}->didErrorOccur)
    {
      $self->setError($self->{dbHandle}->errorMessage);
      return 0;
    }

    # beginning_balance_entry_tb
    $self->{dbHandle}->write(sql => "DELETE FROM beginning_balance_entry_tb WHERE ledger_code = ?", plug => [ $ledgerCode]);
    if ($self->{dbHandle}->didErrorOccur)
    {
      $self->setError($self->{dbHandle}->errorMessage);
      return 0;
    }

    # local_accounts_tb
    $self->{dbHandle}->write(sql => "DELETE FROM local_accounts_tb WHERE ledger_code = ?", plug => [ $ledgerCode]);
    if ($self->{dbHandle}->didErrorOccur)
    {
      $self->setError($self->{dbHandle}->errorMessage);
      return 0;
    }

    # sec_inv_tb
    $self->{dbHandle}->write(sql => "DELETE FROM sec_inv_tb WHERE ledger_code = ?", plug => [ $ledgerCode]);
    if ($self->{dbHandle}->didErrorOccur)
    {
      $self->setError($self->{dbHandle}->errorMessage);
      return 0;
    }

    # sec_event_tb
    $self->{dbHandle}->write(sql => "DELETE FROM sec_event_tb WHERE ledger_code = ?", plug => [ $ledgerCode]);
    if ($self->{dbHandle}->didErrorOccur)
    {
      $self->setError($self->{dbHandle}->errorMessage);
      return 0;
    }

    # dep_sched_tb
    $self->{dbHandle}->write(sql => "DELETE FROM dep_sched_tb WHERE ledger_code = ?", plug => [ $ledgerCode]);
    if ($self->{dbHandle}->didErrorOccur)
    {
      $self->setError($self->{dbHandle}->errorMessage);
      return 0;
    }

    # report_index_tb
    $self->{dbHandle}->write(sql => "DELETE FROM report_index_tb WHERE ledger_code = ?", plug => [ $ledgerCode]);
    if ($self->{dbHandle}->didErrorOccur)
    {
      $self->setError($self->{dbHandle}->errorMessage);
      return 0;
    }

    # report_tb
    $self->{dbHandle}->write(sql => "DELETE FROM report_tb WHERE ledger_code = ?", plug => [ $ledgerCode]);
    if ($self->{dbHandle}->didErrorOccur)
    {
      $self->setError($self->{dbHandle}->errorMessage);
      return 0;
    }

    # memorized_report_tb
    $self->{dbHandle}->write(sql => "DELETE FROM memorized_report_tb WHERE ledger_code = ?", plug => [ $ledgerCode]);
    if ($self->{dbHandle}->didErrorOccur)
    {
      $self->setError($self->{dbHandle}->errorMessage);
      return 0;
    }

    # tax_loss_tb
    $self->{dbHandle}->write(sql => "DELETE FROM tax_loss_tb WHERE ledger_code = ?", plug => [ $ledgerCode]);
    if ($self->{dbHandle}->didErrorOccur)
    {
      $self->setError($self->{dbHandle}->errorMessage);
      return 0;
    }

    # scratch_tb
    $self->{dbHandle}->write(sql => "DELETE FROM scratch_tb WHERE ledger_code = ?", plug => [ $ledgerCode]);
    if ($self->{dbHandle}->didErrorOccur)
    {
      $self->setError($self->{dbHandle}->errorMessage);
      return 0;
    }

    # lot_node_tb
    $self->{dbHandle}->write(sql => "DELETE FROM lot_node_tb WHERE ledger_code = ?", plug => [ $ledgerCode]);
    if ($self->{dbHandle}->didErrorOccur)
    {
      $self->setError($self->{dbHandle}->errorMessage);
      return 0;
    }

    $self->{dbHandle}->commit;
    if ($self->{dbHandle}->didErrorOccur)
    {
      $self->setError($self->{dbHandle}->errorMessage);
      return 0;
    }
  }
  else
  {
    return -1;
  }

  return 1;
}

=item Objects::Ledger[] getLedgers(ledgerCode =>, ledgerName =>)

    Returns an array of Ledger Objects matching the parameters passed.
    No params means select all ledgers.  Empty array is returned if
    no ledgers exist or match criteria.
    NOTE: Will not error on an invalid ledger.  You must do an
    ->isValid on each returned ledgerObj to decide what to do with it.
    Undef returned on errors.

=cut
sub getLedgers
{
  my $self = shift;
  my %args = ( ledgerCode => "", ledgerName => "", @_ ) ;
  my @Ledgers = ();
  my $where = "";

  if ($args{ledgerCode} ne "" || $args{ledgerName} ne "")
  {
    $where .= " AND ";
    if ($args{ledgerCode} ne "" && $args{ledgerName} ne "")
    {
      $where .= " ( ledger_tb.name = '" . $args{ledgerName} . "'";
      $where .= " OR ledger_tb.code = '" . $args{ledgerCode} . "' ) ";
    }
    elsif ($args{ledgerName} ne "")
    {
      $where .= " ledger_tb.name = '" . $args{ledgerName} . "'";
    }
    else
    {
      $where .= " ledger_tb.code = '" . $args{ledgerCode} . "'";
    }
  }

  my $sql = "SELECT ledger_tb.name, ledger_tb.code, ledger_tb.color";
  $sql .= ", ledger_tb.cash_or_accrual ";
  $sql .= ", business_type_tb.name, ledger_tb.business_type ";
  $sql .= ", offset1 ";
  $sql .= ", start1 ";
  $sql .= ", start2 ";
  $sql .= ", start3 ";
  $sql .= ", start4 ";
  $sql .= ", start5 ";
  $sql .= ", start6 ";
  $sql .= ", start7 ";
  $sql .= ", start8 ";
  $sql .= ", start9 ";
  $sql .= ", start10 ";
  $sql .= ", start11 ";
  $sql .= ", start12 ";
  $sql .= "FROM ledger_tb, business_type_tb ";
  $sql .= "WHERE ledger_tb.business_type = business_type_tb.num ".$where;
  $sql .= "ORDER BY ledger_tb.name, ledger_tb.code";
  my $sth = $self->{dbHandle}->read(sql => $sql);
  if ($self->{dbHandle}->didErrorOccur)
  {
    $self->setError($sql."<br>\n". $self->{dbHandle}->errorMessage);
    return undef;
  }
  while (my @info = $sth->fetchrow_array)
  {
    my %knownYears = $self->getKnownYears(ledgerCode => $info[1]);
    if ($self->didErrorOccur)
    {
      $self->setError(" Sent ledger '$args{ledgerCode}' to function getKnownYears<br>\n");
      return undef;
    }
    my $cashOrAccrual = "";
    if ($info[3] =~ /^t|true|1$/)  #!!!!find DB independent way to do this!
    {  $cashOrAccrual = "cash";}
    elsif ($info[3] =~ /^f|false|0$/)
    { $cashOrAccrual = "accrual";}
    else
    { $cashOrAccrual = "error!";}

    my @start = ( $info[6], $info[7], $info[8], $info[9], $info[10], $info[11], $info[12], $info[13], $info[14], $info[15], $info[16], $info[17], $info[18], $info[19]);

    my $ledgerObj = Portal::Accounting::Objects::Ledger->new( langObj => $self->{langObj},
                    name => $info[0], code => $info[1], color => $info[2],
                    cashOrAccrual => $cashOrAccrual,
                    businessType => $info[5],
                    businessTypeName => $info[4],
                    knownYears => \%knownYears,
                    start => \@start,
                    );

    $Ledgers[++$#Ledgers] = $ledgerObj;
  }

  return @Ledgers;
}

=item Objects::Ledger getLedger(ledgerCode =>)

    Returns the Ledger Object matching the specified ledgerCode or
    undef if not found.

    You need to verify the validity of the returned object yourself.

=cut
sub getLedger
{
  my $self = shift;
  my %args = ( ledgerCode => "", @_ ) ;
  my $ledger = undef;
  my $where = "";

  if ($args{ledgerCode} ne "")
  {
    $where .= " AND ";
    $where .= " ledger_tb.code = '" . $args{ledgerCode} . "'";
  }
  else
  {
    $self->missing("ledgerCode");
    $self->postfixError($self->genErrorString("all"));
    return undef;
  }

  my $sql = "SELECT ledger_tb.name, ledger_tb.code, ledger_tb.color";
  $sql .= ", ledger_tb.cash_or_accrual ";
  $sql .= ", business_type_tb.name, ledger_tb.business_type ";
  $sql .= ", offset1 ";
  $sql .= ", start1 ";
  $sql .= ", start2 ";
  $sql .= ", start3 ";
  $sql .= ", start4 ";
  $sql .= ", start5 ";
  $sql .= ", start6 ";
  $sql .= ", start7 ";
  $sql .= ", start8 ";
  $sql .= ", start9 ";
  $sql .= ", start10 ";
  $sql .= ", start11 ";
  $sql .= ", start12 ";
  $sql .= "FROM ledger_tb, business_type_tb ";
  $sql .= "WHERE ledger_tb.business_type = business_type_tb.num ".$where;
  $sql .= "ORDER BY ledger_tb.name, ledger_tb.code";
  my $sth = $self->{dbHandle}->read(sql => $sql);
  if ($self->{dbHandle}->didErrorOccur)
  {
    $self->setError($sql."<br>\n". $self->{dbHandle}->errorMessage);
    return undef;
  }
  my @info = $sth->fetchrow_array;
  if (defined $info[0])
  {
    my %knownYears = $self->getKnownYears(ledgerCode => $info[1]);
    if ($self->didErrorOccur)
    {
      $self->setError(" Sent ledger '$args{ledgerCode}' to function getKnownYears<br>\n");
      return undef;
    }
    my $cashOrAccrual = "";
    if ($info[3] =~ /^t|true|1$/)  #!!!!find DB independent way to do this!
    {  $cashOrAccrual = "cash";}
    elsif ($info[3] =~ /^f|false|0$/)
    { $cashOrAccrual = "accrual";}
    else
    { $cashOrAccrual = "error!";}

    my @start = ( $info[6], $info[7], $info[8], $info[9], $info[10], $info[11], $info[12], $info[13], $info[14], $info[15], $info[16], $info[17], $info[18], $info[19]);

    $ledger = Portal::Accounting::Objects::Ledger->new( langObj => $self->{langObj},
                    name => $info[0], code => $info[1], color => $info[2],
                    cashOrAccrual => $cashOrAccrual,
                    businessType => $info[5],
                    businessTypeName => $info[4],
                    knownYears => \%knownYears,
                    start => \@start,
                    );
  }
  else
  {
    return undef;
  }

  return $ledger;
}

=head2 Hash getBusinessTypeNames()

    This routine selects the business type names from the database and returns them as an hash
    of number = name values where the name is looked up from the Language Object so as to be
    Internationalized.

=cut
sub getBusinessTypeNames
{
  my $self = shift;
  my $errStr = "Portal::Accounting::DB::Ledger->getBusinessTypes() - Error!<br>\n";

  my $index = 0;
  my @info;
  my $sth;
  my %types;

  $sth = $self->{dbHandle}->read(sql => "SELECT name, num FROM business_type_tb ORDER BY num");
  if ($self->{dbHandle}->didErrorOccur)
  {
    $self->{error} = 1;
    $self->{errorString} = $errStr . $self->{dbHandle}->errorMessage;
    return undef;
  }
  while (@info = $sth->fetchrow_array)
  {
    $types{$info[1]} = $self->{langObj}->map($info[0]);
  }

  return %types;
}

=head2 bool isValidData(field => businessType, data => "Corporation", ledger => $ledger)

=cut
sub isValidData
{
  my $self = shift;
  my $errStr = "Portal::Accounting::DB::Ledger->isValidData() - Error!<br>\n";

  return undef;
}

=head2 SubEntityObject getSubEntities(ledger =>)

=cut
sub getSubEntities
{
  my $self = shift;
  my $errStr = "Portal::Accounting::DB::Ledger->getSubEntities() - Error!<br>\n";

  return undef;
}

=head2 bool addSubEntity(ledger=>, code =>, description =>)

=cut
sub addSubEntity
{
  my $self = shift;
  my $errStr = "Portal::Accounting::DB::Ledger->addSubEntity() - Error!<br>\n";

  return undef;
}

=head2 bool deleteSubEntity(ledger=>, code =>, description =>)

=cut
sub deleteSubEntity
{
  my $self = shift;
  my $errStr = "Portal::Accounting::DB::Ledger->deleteSubEntity() - Error!<br>\n";

  return undef;
}

=head2 FiscalYearObject getCurrentFiscalYear(ledgerCode)

     This method returns a FiscalYearObject that holds the current Fiscal Year's info for the specified ledger.

=cut
sub getCurrentFiscalYear
{
  my $self = shift;
  my %args = ( @_ );
  my $ledgerCode = $args{ledgerCode};
  my $fiscalYearObj = undef;
  my $errStr = "Portal::Accounting::DB::Main->getCurrentFiscalYear()  - Error!<br>\n";
  my $sth;
  my @info;

  if (length $ledgerCode == 0)
  {
    $self->{error} = 1;
    $self->{errorString} = $errStr . "ledgerCode = '$ledgerCode' is invalid!<br>\n";
    return $fiscalYearObj;
  }

  my $sql = "SELECT year, closed, start_date, end_date, q1_start, q1_end, q2_start, q2_end, q3_start, q3_end, q4_start, q4_end, ledger_code ";
  $sql .= " FROM known_years_tb WHERE ledger_code = '$ledgerCode' AND ";
  $sql .= " year = (SELECT working_year FROM variables_tb WHERE ledger_code = '$ledgerCode')";
  $sth = $self->{dbHandle}->read(sql => $sql);
  if ($self->{dbHandle}->didErrorOccur)
  {
    $self->{error} = 1;
    $self->{errorString} = $errStr . $self->{dbHandle}->errorMessage;
    return $fiscalYearObj;
  }
  @info = $sth->fetchrow_array;
  if (!defined $info[0])
  {
    $self->{error} = 1;
    $self->{errorString} = $errStr . "No data returned!<br>\n";
    return $fiscalYearObj;
  }
  $fiscalYearObj = Portal::Accounting::Objects::FiscalYearObject->new(year => $info[0], closed => $self->portalVariables->trueFalse->{$info[1]},
                   startDate => $info[2], endDate => $info[3], q1Start => $info[4], q1End => $info[5], q2Start => $info[6], q2End => $info[7],
                   q3Start => $info[8], q3End => $info[9], q4Start => $info[10], q4End => $info[11], ledgerCode => $info[12]);
  if ($fiscalYearObj->didErrorOccur)
  {
    $self->{error} = 1;
    $self->{errorString} = $errStr . $fiscalYearObj->errorMessage;
    return undef;
  }

  return $fiscalYearObj;
}

=head2 FiscalYearObject[] getAllKnownYears(ledgerCode)

    This method returns an array of all Fiscal Years for the specified ledger in oldest to newest order.

=cut
sub getAllKnownYears
{
  my $self = shift;
  my %args = ( @_ );
  my $ledgerCode = $args{ledgerCode};
  my %knownYears = ();
  my $errStr = "Portal::Accounting::DB::Main->getAllKnownYears()  - Error!<br>\n";
  my $sth;
  my @info;

  if (length $ledgerCode == 0)
  {
    $self->{error} = 1;
    $self->{errorString} = $errStr . "ledgerCode = '$ledgerCode' is invalid!<br>\n";
    return undef;
  }

  $sth = $self->{dbHandle}->read(sql => "SELECT year, end_date FROM known_years_tb WHERE ledger_code = '$ledgerCode' ORDER BY year");
  if ($self->{dbHandle}->didErrorOccur)
  {
    $self->{error} = 1;
    $self->{errorString} = $errStr . $self->{dbHandle}->errorMessage;
    return undef;
  }

  while (@info = $sth->fetchrow_array)
  {
    $knownYears{$info[0]} = $info[1];
  }
  if (scalar keys %knownYears == 0)
  {
    $self->{error} = 1;
    $self->{errorString} = $errStr . "No data returned!<br>\n";
    return undef;
  }

  return %knownYears;
}

=head2 %years getKnownYears(ledgerCode =>)

    required: ledgerCode
    Returns hash of years as keys with YYYY/MM/DD as data for each key

=cut
sub getKnownYears
{
  my $self = shift;
  my %args = ( ledgerCode => "", @_ );
  my %knownYears = ();
  my $years = 0;

  if (not defined $args{ledgerCode})
  {
    $self->setError("Error! ledgerCode is not defined!<br>\n");
    return undef;
  }
  if ($args{ledgerCode} eq "")
  {
    $self->setError("Error! ledgerCode = '' is invalid!<br>\n");
    return undef;
  }
  my $sth = $self->{dbHandle}->write(sql => "SET datestyle TO 'ISO'");
  if ($self->{dbHandle}->didErrorOccur)
  {
    $self->setError("Error setting datestyle to ISO!<br>\n" . $self->{dbHandle}->errorMessage);
    return undef;
  }
  my $select = "SELECT year, q4_end from known_years_tb where ledger_code = '$args{ledgerCode}'";
  $sth = $self->{dbHandle}->read(sql => $select);
  if ($self->{dbHandle}->didErrorOccur)
  {
    $self->setError("Error in getKnownYears!<br>\n" . $self->{dbHandle}->errorMessage);
    return undef;
  }
  while (my @info = $sth->fetchrow_array)
  {
    $knownYears{$info[0]} = $info[1];
    $years++;
  }
  if ($years == 0)
  {
    $self->setError( "Error in getKnownYears! No known years returned from database for ledger code '$args{ledgerCode}'.<br>\n");
    return undef;
  }
  return %knownYears;
}

1;
__END__

=

  NOTE:  All data fields are accessible by specifying the object
         and pointing to the data member to be modified on the
         left-hand side of the assignment.
         Ex.  $obj->variable($newValue); or $value = $obj->variable;


=head1 AUTHOR

JT Moree, Xperience, Inc. <moreejt@pcxperience.org>

=head1 SEE ALSO

perl, Portal::Accounting

=cut

