# WatchList.pm - The Object Class that does the work of creating a
# Finance::Quote object, fetching the data for the list of watched stocks, and
# formatting output for a table (method called getWatchList)

# Copyright (c) 2000, 2002 PC & Web Xperience, Inc. http://www.pcxperience.com/
# All rights reserved.  This program is free software; you can redistribute it
# and/or modify it under the same terms of the GPL

package Portal::Accounting::Helpers::WatchList;
use strict;

use Finance::Quote;
use POSIX;
use HTMLObject::Normal;

use Portal::Accounting::DB::Security;
use Portal::Accounting::DisplayItems;
use vars qw($AUTOLOAD $VERSION @ISA @EXPORT);

require Exporter;

@ISA = qw(Exporter AutoLoader);
@EXPORT = qw();

$VERSION = '$Revision: 1.14 $';

=item new ( dbHandle => , configObj => , langObj => , 'DB::Security' => )

 instantiates an instance of the WatchList Helper Object
 requires: dbHandle, configObj, langObj
 optional: 'DB::Security' already existing Object with a valid DB handle

=cut
sub new
{
  my $that = shift;
  my $class = ref($that) || $that;
  my $self = bless {}, $class;
  my %args = ( dbHandle => undef, 'DB::Security' => undef, configObj => undef, langObj => undef, @_ );
  my $errStr = "Portal::Accounting::Helpers::WatchList->new() - Error!<br>\n";

  $self->{dbHandle} = $args{dbHandle};
  $self->{configObj} = $args{configObj};
  $self->{langObj} = $args{langObj};
  # instantiate Portal::Accounting::DB::Security object here
  $self->{dbSecurityObj} = $args{'DB::Security'};

  # hardcode (for now) the financial website to use for ticker popups
  $self->{tickerURL} = "http://finance.yahoo.com/q?s=%s";

  if ((not defined $self->{dbSecurityObj}) || $self->{dbSecurityObj}->didErrorOccur)
  {
    $self->{dbSecurityObj} = Portal::Accounting::DB::Security->new(langObj => $self->{langObj}, dbHandle => $self->{dbHandle});
  }
  $self->{displayObj} = Portal::Accounting::DisplayItems->new;
  # instantiate Finance::Quote object here
  $self->{financeQuoteObj} = Finance::Quote->new();

  # do validation
  $self->{error} = !$self->isValid;
  if ($self->{error})
  {
    $self->{errorString} = $errStr . $self->{errorString};
  }

  return $self;
}

# isValid - Returns 0 or 1 to indicate if the object is valid.
sub isValid
{
  my $self = shift;
  my $error = 0;
  my $errorString = "";
  my $errStr = "Portal::Accounting::Helpers::WatchList->isValid() - Error!<br>\n";

  # do validation code here.
  if (not defined $self->{dbHandle})
  {
    $error = 1;
    $errorString .= "dbHandle must be defined!<br>\n";
  }
  if (not defined $self->{configObj})
  {
    $error = 1;
    $errorString .= "configObj must be defined!<br>\n";
  }
  if (not defined $self->{langObj})
  {
    $error = 1;
    $errorString .= "langObj must be defined!<br>\n";
  }
  if ($self->{dbSecurityObj}->didErrorOccur)
  {
    $error = 1;
    $errorString .= "Error initializing dbSecurityObj object!<br>\n";
    $errorString .= $self->{dbSecurityObj}->errorMessage;
  }
  if (not defined $self->{financeQuoteObj})
  {
    $error = 1;
    $errorString .= "Error instantiating Finance::Quote object!<br>\n";
  }

  $self->{error} = $error;
  $self->{errorString} = $errStr if $error;
  $self->{errorString} .= $errorString;

  return !$error;
}

sub DESTROY
{
  my $self = shift;
}

sub AUTOLOAD
{
  my $self = shift;
  my $type = ref($self) || die "$self is not an object";
  my $name = $AUTOLOAD;
  $name =~ s/.*://;	# strip fully-qualified portion
  unless (exists $self->{$name})
  {
    die "Can't access `$name' field in object of class $type";
  }
  if (@_)
  {
    return $self->{$name} = shift;
  }
  else
  {
    return $self->{$name};
  }
}

# setError
# requires: errorString
# returns: nothing
sub setError
{
  my $self = shift;
  my %args = ( errorString => "", @_ );
  my $errorString = $args{errorString};

  $self->{errorString} = $errorString;
  $self->{error} = 1;
}

# didErrorOccur
# returns the value of error.
sub didErrorOccur
{
  my $self = shift;

  return $self->{error};
}

# errorMessage
# returns the value of errorString
sub errorMessage
{
  my $self = shift;

  return $self->{errorString};
}

# getWatchList
# takes: ledgerCode
# returns: HTML <table> output in the event of success; undef if
# something fails
# This routine queries the database for the list of watched stocks.  If
# no stocks exist in the watched stocks table, output "No stocks are
# currently being watched!", but do not return an error condition.  If
# individual but not all stocks fail to be fetched, only return those
# stocks that were successfully fetched, and indicate those that failed
# in a small NOTE at the bottom of the table output.
sub getWatchList
{
  my $self = shift;
  my %args = ( ledgerCode => undef, ledger => undef, @_ );
  my $ledgerCode = $args{ledgerCode};
  my $ledger = $args{ledger};
  my $errStr = "Portal::Accounting::Helpers::WatchList->getWatchList() - Error!<br>\n";
  my $htmlString = "";

  # validate input
  if (length $ledgerCode == 0)
  {
    $self->setError(errorString => $errStr . "ledgerCode = '$ledgerCode' is invalid!<br>\n");
    return undef;
  }
  if (length $ledger == 0)
  {
    $self->setError(errorString => $errStr . "ledger = '$ledger' is invalid!<br>\n");
    return undef;
  }

  # get list of watched stocks from database with ticker symbols returned
  my %watchList = $self->{dbSecurityObj}->getListOfWatchedStocks(orderBy => "name", ledgerCode => $ledgerCode, ticker => "yes", byGroupName => "2");
  if ($self->{dbSecurityObj}->didErrorOccur)
  {
    $self->setError(errorString => $errStr . $self->{dbSecurityObj}->errorMessage);
    return undef;
  }

  if ((scalar keys %{$watchList{stocks}}) eq "0")
  {
    $htmlString = "<b><i>" . $self->{langObj}->map("noStocksReturned") . "</i></b><br>" . $self->{langObj}->map("pleaseWatchStocks") . "\n";
    return $htmlString;
  }

  # set the list of required labels that must exist for any stock quotes returned
  $self->{financeQuoteObj}->require_labels("name", "last", "net", "p_change", "volume", "date");

  # build an array from the watchList hash
  my @watchListArray = keys %{$watchList{stocks}};

  # fetch the information for the ticker symbols using Finance::Quote module
  # hardcode the use of the "usa" method for now
  my %quotes = $self->{financeQuoteObj}->fetch("usa", @watchListArray);

  # if nothing returned from the fetch, return an error message, but do not set an error condition
  if (not %quotes)
  {
    $htmlString = "<b><i>" . $self->{langObj}->map("noInformationReturnedFromFetch") . "</i></b>\n";
    return $htmlString;
  }

  # run through fetch results and output a row of information for each successfully
  # returned result ordered by company name
  my $failedTickers = "";
  my $date = POSIX::strftime('%m/%d/%Y', localtime);
  (my $localMonth, my $localDay, my $localYear) = split '/', $date, 3;
  my $oddCounter = 1;
  my $currentGroup = "";
  my $oldGroup = "";
  my $colorClass = "";
  my $totalGL = 0;
  my $totalCost = 0;
  my $totalMkt = 0;
  my $classGainLoss = "WLnoGainLossChange";
  # begin outer table
  $htmlString .= "<table cellspacing=\"2\" cellpadding=\"2\" border=\"0\">\n";
  $htmlString .= "  <tr>\n";
  $htmlString .= "    <td>\n";
  foreach my $dataRow (@{$watchList{dataRows}})
  {
    my $groupName = @{$dataRow}[0];
    my $ticker = @{$dataRow}[1];

    $currentGroup = $groupName;

    if (($oldGroup ne $currentGroup) && ($oldGroup ne ""))
    {
      # sub-totals
      $htmlString .= <<"EOF";
    <tr>
      <td>&nbsp;</td><td>&nbsp;</td><td>&nbsp;</td><td>&nbsp;</td>
      <td>&nbsp;</td><td>&nbsp;</td><td>&nbsp;</td><td>&nbsp;</td><td>&nbsp;</td>
EOF
      $htmlString .= "      <td class=\"WLtotalMarketValue\">";
      $htmlString .= $self->{displayObj}->commify(value => sprintf("%.0f", $totalMkt)) . "</td>\n";
      $htmlString .= "      <td class=\"WLtotalCost\">";
      $htmlString .= $self->{displayObj}->commify(value => sprintf("%.0f", $totalCost)) . "</td>\n";
      $classGainLoss = "WLnoGainLossChange";
      if ($totalGL > 0.0)
      {
        $classGainLoss = "WLgain";
      }
      elsif ($totalGL < 0.0)
      {
        $classGainLoss = "WLloss";
      }
      $htmlString .= "      <td class=\"$classGainLoss"."Total\">";
      $htmlString .= $self->{displayObj}->commify(value => sprintf("%.0f", $totalGL)) . "</td>\n";
      $htmlString .= <<"EOF";
      </td><td>&nbsp;</td>
    </tr>
EOF
      # reset key counters
      $totalGL = 0;
      $totalCost = 0;
      $totalMkt = 0;
      # close the table; this step will also have to be done outside the end of the loop
      $htmlString .= "</table>\n";
      # add the list of failed ticker lookups, if any, to the end of the table output
      if ($failedTickers)
      {
        $htmlString .= "<br><br>" . $self->{langObj}->map("tickersFailedToLookup") . ": <b>$failedTickers</b>\n";
	# clear failedTickers for the next group
	$failedTickers = "";
      }
      $htmlString .= "</span>\n";
      # maybe add a separator of some kind between the group outputs?  i.e., <hr>
      $htmlString .= "<hr noshade size=\"1\">\n";

      # toggle oddCounter
      $oddCounter = not $oddCounter;
    }

    if ($currentGroup ne $oldGroup)
    {
      $colorClass = "WL" . (($oddCounter) ? "odd" : "even") . "Group";
      my $d = HTMLObject::Base->new;
      if ($d->didErrorOccur)
      {
        $self->setError(errorString => $errStr . $d->errorMessage);
        return undef;
      }
      my $InvW = $d->encodeString(string => "Investments::WatchList");
      my $lc = $d->encodeString(string => $ledgerCode);
      my $l = $d->encodeString(string => $ledger);
      $htmlString .= <<"EOF";
<span class="$colorClass">
<center><a href="?app=Accounting&state=$InvW&command=editWatchList&groupName=$groupName&ledger=$l&ledgerCode=$lc"><span class="WLgroupName">$groupName</span></a></center>
<table class="WLquoteTable $colorClass" width="100\%" border="0" cellspacing="2" cellpadding="2">
EOF
      $htmlString .= "  <tr><td class=\"WLcompanyNameLabel\"><b>" . $self->{langObj}->map("Company") . "</b></td>";
      $htmlString .= "    <td class=\"WLtickerSymbolLabel\"><b>" . $self->{langObj}->map("Ticker") . "</b></td>";
      $htmlString .= "    <td align=\"right\" class=\"WLtotalSharesLabel\"><b>" . $self->{langObj}->map("totalShares") . "</b></td>";
      $htmlString .= "    <td align=\"right\" class=\"WLaverageCostPerShareLabel\"><b>" . $self->{langObj}->map("avgCostPerShare") . "</b></td>";
      $htmlString .= "    <td align=\"right\" class=\"WLlastPriceLabel\"><b>" . $self->{langObj}->map("Price") . "</b></td>";
      $htmlString .= "    <td align=\"right\" class=\"WLnetChangeLabel\"><b>" . $self->{langObj}->map("Change") . "</b></td>";
      $htmlString .= "    <td>&nbsp;</td>";
      $htmlString .= "    <td align=\"right\" class=\"WLpercentChangeLabel\"><b>" . $self->{langObj}->map("percentChange") . "</b></td>";
      $htmlString .= "    <td align=\"right\" class=\"WLvolumeTradedLabel\"><b>" . $self->{langObj}->map("volumeTraded") . "</b></td>";
      $htmlString .= "    <td align=\"right\" class=\"WLtotalMarketValueLabel\"><b>" . $self->{langObj}->map("Mkt Val") . "</b></td>";
      $htmlString .= "    <td align=\"right\" class=\"WLtotalCostLabel\"><b>" . $self->{langObj}->map("totalCost") . "</b></td>";
      $htmlString .= "    <td align=\"right\" class=\"WLgainLossLabel\"><b>" . $self->{langObj}->map("gainLoss") . "</b></td>";
      $htmlString .= "    <td>&nbsp;</td>";
      $htmlString .= "  </tr>\n";
    }

    # build a list of failed ticker symbols to output below the table of results
    if ((exists $quotes{$ticker}) && (not $quotes{$ticker, "success"}))
    {
      if ($failedTickers)
      {
        $failedTickers .= ", ";
      }
      $failedTickers .= $ticker;
    }
    else
    {
      $htmlString .= "  <tr>\n";
      # company name
      #my $name = ucfirst(lc($quotes{$ticker, "name"}));
      my $returnedName = $watchList{stocks}{$ticker}{name};
      $htmlString .= "    <td class=\"WLcompanyName\">$returnedName</td>";
      # ticker symbol (in parenthesis) with link to popup window
      my $substitutedTickerURL = sprintf($self->{tickerURL}, $ticker);
      $htmlString .= "<td class=\"WLtickerSymbol\"><a href=\"$substitutedTickerURL\" target=\"tickerPopUpWindow$ticker\">$ticker</a></td>";
      # total shares for stock
      $htmlString .= "<td class=\"WLtotalShares\">" . $self->{displayObj}->commify(value => sprintf("%.0f", $watchList{stocks}{$ticker}{totalShares})) . "</td>";
      # average cost per share for stock
      $htmlString .= "<td class=\"WLaverageCostPerShare\">" . $self->{displayObj}->commify(value => sprintf("%.2f", $watchList{stocks}{$ticker}{avgCostPerShare})) . "</td>";
      # last traded price
      $htmlString .= "<td class=\"WLlastPrice\">" . $self->{displayObj}->commify(value => $quotes{$ticker, "last"}) . "</td>";
      # net change (color coded)
      if ($quotes{$ticker, "net"} > 0)
      {
        $htmlString .= "<td class=\"WLnetIncrease\">" . $self->{displayObj}->commify(value => $quotes{$ticker, "net"}) . "</td>";
      }
      elsif ($quotes{$ticker, "net"} < 0)
      {
        $htmlString .= "<td class=\"WLnetDecrease\">" . $self->{displayObj}->commify(value => $quotes{$ticker, "net"}) . "</td>";
      }
      else
      {
        $htmlString .= "<td class=\"WLnetChange\">" . $self->{displayObj}->commify(value => $quotes{$ticker, "net"}) . "</td>";
      }
      # determine if there needs to be an arrow image and in what direction
      if ($quotes{$ticker, "net"} > 0)
      {
        $htmlString .= "<td class=\"WLnetIncreaseImage\"><img src=\"". $self->{configObj}->siteDir . "/images/Accounting/stock_increase.gif\" border=\"0\"></img></td>";
      }
      elsif ($quotes{$ticker, "net"} < 0)
      {
        $htmlString .= "<td class=\"WLnetDecreaseImage\"><img src=\"". $self->{configObj}->siteDir . "/images/Accounting/stock_decrease.gif\" border=\"0\"></img></td>";
      }
      else
      {
        # empty table cell
        $htmlString .= "<td>&nbsp;</td>";
      }
      # percent change (color coded)
      if ($quotes{$ticker, "p_change"} > 0)
      {
        $htmlString .= "<td class=\"WLpercentIncrease\">" . $quotes{$ticker, "p_change"} . "&#37;</td>";
      }
      elsif ($quotes{$ticker, "p_change"} < 0)
      {
        $htmlString .= "<td class=\"WLpercentDecrease\">" . $quotes{$ticker, "p_change"} . "&#37;</td>";
      }
      else
      {
        $htmlString .= "<td class=\"WLpercentChange\">" . $quotes{$ticker, "p_change"} . "</td>";
      }
      # volume traded
      $htmlString .= "<td class=\"WLvolumeTraded\">" . $self->{displayObj}->commify(value => sprintf("%.01f", $quotes{$ticker, "volume"}/1000000) ) . "</td>";
      # total market value
      my $marketValue = $watchList{stocks}{$ticker}{totalShares} * $quotes{$ticker, "last"};
      $htmlString .= "<td class=\"WLmarketValue\">" . $self->{displayObj}->commify(value => sprintf("%.0f", $marketValue)) . "</td>";
      $totalMkt += $marketValue;
      # total cost
      my $cost = $watchList{stocks}{$ticker}{totalCost};
      $htmlString .= "<td class=\"WLcost\">" . $self->{displayObj}->commify(value => sprintf("%.0f", $cost)) . "</td>";
      $totalCost += $cost;
      # gain/loss
      my $gainLoss = $marketValue - $cost;
      $totalGL += $gainLoss;
      $classGainLoss = "WLnoGainLossChange";
      if ($gainLoss > 0.0)
      {
        $classGainLoss = "WLgain";
      }
      elsif ($gainLoss < 0.0)
      {
        $classGainLoss = "WLloss";
      }
      $htmlString .= "<td class=\"$classGainLoss\">" . $self->{displayObj}->commify(value => sprintf("%.0f", $gainLoss)) . "</td>";
      # date (only displayed if different from today's date, otherwise add empty table cell)
      my $quoteDate = $quotes{$ticker, "date"};
      (my $quoteMonth, my $quoteDay, my $quoteYear) = split '/', $quoteDate, 3;
      if (($quoteMonth == $localMonth) and ($quoteDay == $localDay) and ($quoteYear == $localYear))
      {
        $htmlString .= "<td>&nbsp;</td>";
      }
      else
      {
        $htmlString .= "<td class=\"WLquoteDate\">$quoteDate ($date)</td>";
      }
      $htmlString .= "\n  </tr>\n";
    }

    # set the oldGroup to the currentGroup for the next go around the loop
    $oldGroup = $currentGroup;
  }
  # sub-totals
  $htmlString .= <<"EOF";
    <tr>
      <td>&nbsp;</td><td>&nbsp;</td><td>&nbsp;</td><td>&nbsp;</td>
      <td>&nbsp;</td><td>&nbsp;</td><td>&nbsp;</td><td>&nbsp;</td><td>&nbsp;</td>
EOF
  $htmlString .= "      <td class=\"WLtotalMarketValue\">";
  $htmlString .= $self->{displayObj}->commify(value => sprintf("%.0f", $totalMkt)) . "</td>\n";
  $htmlString .= "      <td class=\"WLtotalCost\">";
  $htmlString .= $self->{displayObj}->commify(value => sprintf("%.0f", $totalCost)) . "</td>\n";
  $classGainLoss = "WLnoGainLossChange";
  if ($totalGL > 0.0)
  {
    $classGainLoss = "WLgain";
  }
  elsif ($totalGL < 0.0)
  {
    $classGainLoss = "WLloss";
  }
  $htmlString .= "      <td class=\"$classGainLoss"."Total\">";
  $htmlString .= $self->{displayObj}->commify(value => sprintf("%.0f", $totalGL)) . "</td>\n";
  $htmlString .= <<"EOF";
      </td><td>&nbsp;</td>
    </tr>
  </table>
EOF
  # add the list of failed ticker lookups, if any, to the end of the table output
  if ($failedTickers)
  {
    $htmlString .= "<br><br>" . $self->{langObj}->map("tickersFailedToLookup") . ": <b>$failedTickers</b>\n";
  }
  $htmlString .= "</span>\n";
  # close outer table
  $htmlString .= "    </td>\n";
  $htmlString .= "  </tr>\n";
  $htmlString .= "</table>\n";

  return $htmlString;
}

1;
__END__

=head1 NAME

WatchList - Object used to build a WatchList Class.

=head1 SYNOPSIS

  use Portal::Accounting::Helpers::WatchList;
  my $obj = Portal::Accounting::Helpers::WatchList->new(dbHandle => $dbHandle);
  if ($obj->didErrorOccur())
  {
    die $obj->errorMessage();
  }
  $obj->run(command => $command);

=head1 DESCRIPTION

WatchList is a WatchList Helper Object that retrieves a list of
watched stocks from the database and fetches their current
(delayed) information from the Internet using the Finance::Quote
module.

=head1 Exported FUNCTIONS

  scalar new(dbHandle)
    Creates a new instance of the Portal::Accounting::Helpers::WatchList
    object.

  bool isValid(void)
    Returns 0 or 1 to indicate if the object is valid.  The error will be
    available via errorMessage().

  void setError(errorString)
    Sets error = 1 and errorString = errorString.

  scalar didErrorOccur(void)
    Returns the value of error.

  scalar errorMessage(void)
    Returns the value of errorString.

  scalar getWatchList(ledgerCode)
    Returns an HTML table with all stocks that were successfully
    fetched; if no stocks were available or a drastic error occured,
    undef is returned.

  NOTE:  All data fields are accessible by specifying the object
         and pointing to the data member to be modified on the
         left-hand side of the assignment.
         Ex.  $obj->variable($newValue); or $value = $obj->variable;

=head1 AUTHOR

PC & Web Xperience, Inc. (mailto:admin@pcxperience.com)

=head1 SEE ALSO

perl(1), Portal(3), Portal::App(3)

=cut
