# Journal.pm - The Object Class that provides a Journal Object
#$Id: Journal.pm,v 1.1 2003/07/29 20:30:54 pcxuser Exp $
#This package is released under the GNU General Public License.
#You should have recieved a copy of the GPL with it.
# Copyright (c) 2003 HLR http://www.pcxperience.org  All rights reserved.

package Portal::Accounting::Objects::Journal;
use strict;
use Portal::Accounting::Objects::Base;
use vars qw($AUTOLOAD $VERSION @ISA @EXPORT);

require Exporter;

@ISA = qw(Portal::Accounting::Objects::Base Exporter AutoLoader);
@EXPORT = qw();

$VERSION = '0.01';

=head1 NAME

Journal - Object used to build a Journal Object Class.

=head1 SYNOPSIS

  use Portal::Accounting::Objects::Journal;
  my $JournalObj = Portal::Accounting::Objects::Journal->new(   );
  if ($JournalObj->didErrorOccur())
  {
    die $JournalObj->errorMessage();
  }

=head1 DESCRIPTION

Journal is a Journal class.

=head1 Exported FUNCTIONS

B<NOTE>: I<bool> = 1(true), 0(false)

=over 4

=item scalar new(name, code)

    Creates a new instance of Portal::Accounting::Objects::Journal.
    See Portal::Accounting::Objects::Base(3) for a listing of required arguments.

  requires:
    name
    code

=cut

sub new
{
  my $class = shift;
  my $self = $class->SUPER::new(@_);
  my %args = ( name => "", code => "", @_ );

  if ($self->didErrorOccur)
  {
    $self->prefixError();
    return $self;
  }

  # instantiate anything unique to this module
  $self->{name} = $args{name};
  $self->{code} = $args{code};

  # do validation
  if (!$self->Portal::Accounting::Objects::Journal::isValid)
  {
    # the error is set in the isValid() method.
    return $self;
  }

  # do anything else you might need to do.

  return $self;
}

=item bool isValid(void)

 Returns 0 or 1 to indicate if the object is valid.
 The error will be available via errorMessage().

=cut

sub isValid
{
  my $self = shift;

  # make sure our Parent class is valid.
  if (!$self->SUPER::isValid())
  {
    $self->prefixError();
    return 0;
  }

  # validate our parameters.
  if ($self->{name} eq "")
  {
    $self->invalid("name", $self->{name});
  }
  if ($self->{code} eq "")
  {
    $self->invalid("code", $self->{code});
  }

  if ($self->numInvalid() > 0 || $self->numMissing() > 0)
  {
    $self->postfixError($self->genErrorString("all"));
    return 0;
  }

  return 1;
}

=item string print()

  This routine returns a text string that represents the data in this object.

=cut

=item string printXML(fullXML bool, indent int)

 This routine returns an XML snippet for this object for use in such
 things as exporting data

 fullXML - defaults to 0 (false).  If true, we output the xml header tag.
 indent - defaults to 2.  Specifies how many spaces in our top level tag
          needs to be.  If fullXML is true, then we force 0, otherwise we
          use what the user specifies.

=cut

sub printXML
{
  my $self = shift;
  my $result = "";
  my %args = ( fullXML => 0, indent => 2, @_ );
  my $fullXML = $args{fullXML};
  my $indent = ($fullXML ? 0 : $args{indent});  # start at 0 indentation if we are making a full XML document.
  my $spaceStr = " " x $indent;

  # fixup to meet your actual needs.

  if ($self->isValid())
  {
    if ($fullXML)
    {
      $result .= "<?xml version=\"1.0\" encoding=\"ISO-8859-1\"?>";
    }
    $result .= $spaceStr . "<journal>\n";
    foreach my $name (qw{name code})
    {
      $result .= $spaceStr . "  <$name>$self->{$name}</$name>\n";
    }
    $result .= $spaceStr . "</journal>\n";
  }
  else
  {
    $result .= "Data not valid!\n\n";
    $result .= $self->errorMessage();
    die $result;
  }

  return $result;
}

=item string printHTML()

  This routine returns a string of html that represents this object.

=cut

=item string printForm( prefix => )

  This routine returns xhtml form elements that can be used to edit this object.
  A prefix can be used to uniquely identify this form from others of the same 
  type.  The names of form elements can be obtained in a hash:

  use Portal::Accounting::Objects::Base;
  my %elements = %Portal::Accounting::Objects::Journal::Elements;

  !!Make a copy of the hash as shown above--NOT a refence.
  You would generally place your input from the user within the Elements hash.
  Then you can pass the Elements hash to the new method
  of a Journal object instead of individual arguments.

  foreach my $prefix (%prefixes)
    foreach $index (key %Elements)
      $tempHash{$index} = $input->{$prefix . $index}
    my object = Portal::Accounting::Objects::Journal->new(%tempHash);

=cut

=back

=cut

1;
__END__

=head1 NOTE

 All data fields are accessible by specifying the object
 and pointing to the data member to be modified on the
 left-hand side of the assignment.
 Ex.  $obj->variable($newValue); or $value = $obj->variable;

=head1 AUTHOR

Xperience, Inc. (mailto:xiwa@pcxperience.com)

=head1 SEE ALSO

perl(1), Portal::Accounting::Objects::Base(3)

=cut
