# LedgerContainer.pm - The Object Class that provides a LedgerContainer Object
#$Id: LedgerContainer.pm,v 1.6 2003/07/31 21:46:36 pcxuser Exp $
#This package is released under the GNU General Public License.
#You should have recieved a copy of the GPL with it.
# Copyright (c) 2003 HLR http://www.pcxperience.org  All rights reserved.

package Portal::Accounting::Objects::LedgerContainer;
use strict;
use Portal::Accounting::Objects::Base;
use vars qw($AUTOLOAD $VERSION @ISA @EXPORT);

require Exporter;

@ISA = qw(Portal::Accounting::Objects::Base Exporter AutoLoader);
@EXPORT = qw();

$VERSION = '0.01';

=head1 NAME

LedgerContainerObject - Object used to build a LedgerContainer Object Class.

=head1 SYNOPSIS

  use Portal::Accounting::Objects::LedgerContainer;
  my $LedgerContainerObj = Portal::Accounting::Objects::LedgerContainer->new(   );
  if ($LedgerContainerObj->didErrorOccur())
  {
    die $LedgerContainerObj->errorMessage();
  }

=head1 DESCRIPTION

LedgerContainerObject is a LedgerContainer class.

=head1 Exported FUNCTIONS

B<NOTE>: I<bool> = 1(true), 0(false)

=over 4

=item scalar new( )

    Creates a new instance of Portal::Accounting::Objects::LedgerContainer.
    See Portal::Accounting::Objects::Base(3) for a listing of required arguments.

=cut

sub new
{
  my $class = shift;
  my $self = $class->SUPER::new(@_);
  my %args = ( @_ );

  if ($self->didErrorOccur)
  {
    $self->prefixError();
    return $self;
  }

  # instantiate anything unique to this module
  $self->{ledger} = undef;
  $self->{subEntities} = [];
  $self->{journals} = [];
  $self->{journalEntries} = [];
  $self->{errorCodes} = {
    0  => "%s undefined",
    1  => "%s must be specified",
    2  => "%s = '%s' is invalid",
    3  => "no %s defined",
    4  => "%s: %s = '%s' already defined",
    5  => "%s invalid.  Error = '%s'",
  };

  # do anything else you might need to do.

  return $self;
}

=item bool isValid(void)

 Returns 0 or 1 to indicate if the object is valid.
 The error will be available via errorMessage().

=cut

sub isValid
{
  my $self = shift;
  my @errors = ();

  # validate our parameters.
  if (!defined $self->{ledger})
  {
    push @errors, sprintf($self->{errorCodes}->{0}, 'ledger');
  }
  elsif (!$self->{ledger}->isValid())
  {
    push @errors, sprintf($self->{errorCodes}->{5}, "ledger", $self->{ledger}->errorMessage);
  }
  if (scalar @{$self->{subEntities}} > 0)
  {
    # validate subEntities
    my %encounteredSubEntities = ();
    foreach my $subEntity (@{$self->{subEntities}})
    {
      if (!$subEntity->isValid())
      {
        push @errors, sprintf($self->{errorCodes}->{5}, "subEntity", $subEntity->errorMessage);
      }
      # finish up processing the subEntity.
      if (exists $encounteredSubEntities{$subEntity->{name}})
      {
        push @errors, sprintf($self->{errorCodes}->{4}, "subEntity", "name", $subEntity->{name});
      }
      elsif ($encounteredSubEntities{$subEntity->{name}}->{number} eq $subEntity->{number})
      {
        push @errors, sprintf($self->{errorCodes}->{4}, "subEntity", "number", $subEntity->{number});
      }
      elsif ($encounteredSubEntities{$subEntity->{name}}->{code} eq $subEntity->{code})
      {
        push @errors, sprintf($self->{errorCodes}->{4}, "subEntity", "code", $subEntity->{code});
      }
      else
      {
        # first make sure the number hasn't been duplicated under a different name.
        my $found = 0;
        foreach my $tmpSubEntity (keys %encounteredSubEntities)
        {
          if ($subEntity->{number} eq $encounteredSubEntities{$tmpSubEntity}->{number})
          {
            push @errors, sprintf($self->{errorCodes}->{4}, "subEntity", "number", $subEntity->{number});
            $found = 1;
            last;
          }
          if ($subEntity->{code} eq $encounteredSubEntities{$tmpSubEntity}->{code})
          {
            push @errors, sprintf($self->{errorCodes}->{4}, "subEntity", "code", $subEntity->{code});
            $found = 1;
            last;
          }
        }
        $encounteredSubEntities{$subEntity->{name}} = $subEntity if (!$found);
      }
    }
  }
  if (scalar @{$self->{journals}} > 0)
  {
    # validate journals
    my %encounteredJournals = ();
    foreach my $journal (@{$self->{journals}})
    {
      if (!$journal->isValid())
      {
        push @errors, sprintf($self->{errorCodes}->{5}, "journal", $journal->errorMessage);
      }
      # finish up processing the journal.
      if (exists $encounteredJournals{$journal->{name}})
      {
        push @errors, sprintf($self->{errorCodes}->{4}, "journal", "name", $journal->{name});
      }
      elsif ($encounteredJournals{$journal->{name}}->{code} eq $journal->{code})
      {
        push @errors, sprintf($self->{errorCodes}->{4}, "journal", "code", $journal->{code});
      }
      else
      {
        # first make sure the number hasn't been duplicated under a different name.
        my $found = 0;
        foreach my $tmpJournal (keys %encounteredJournals)
        {
          if ($journal->{code} eq $encounteredJournals{$tmpJournal}->{code})
          {
            push @errors, sprintf($self->{errorCodes}->{4}, "journal", "code", $journal->{code});
            $found = 1;
            last;
          }
        }
        $encounteredJournals{$journal->{name}} = $journal if (!$found);
      }
    }
  }
  if (scalar @{$self->{journalEntries}} > 0)
  {
    # validate journalEntries
    foreach my $journalEntry (@{$self->{journalEntries}})
    {
      if (!$journalEntry->isValid())
      {
        push @errors, sprintf($self->{errorCodes}->{5}, "journalEntry", $journalEntry->errorMessage);
      }
    }
  }

  return ((scalar @errors > 0 ? 0 : 1), \@errors);
}

=item string print()

  This routine returns a text string that represents the data in this object.

=cut

sub print
{
  my $self = shift;
  my $result = "";

  return $result;
}

=item string printXML(fullXML bool, indent int)

 This routine returns an XML snippet for this object for use in such
 things as exporting data

 fullXML - defaults to 0 (false).  If true, we output the xml header tag.
 indent - defaults to 2.  Specifies how many spaces in our top level tag
          needs to be.  If fullXML is true, then we force 0, otherwise we
          use what the user specifies.

=cut

sub printXML
{
  my $self = shift;
  my $result = "";
  my %args = ( fullXML => 0, indent => 2, @_ );
  my $fullXML = $args{fullXML};
  my $indent = ($fullXML ? 0 : $args{indent});  # start at 0 indentation if we are making a full XML document.
  my $spaceStr = " " x $indent;

  my @valid = $self->isValid();
  if ($valid[0])
  {
    $result .= "<?xml version=\"1.0\" encoding=\"ISO-8859-1\"?>" if ($fullXML);
    $result .= $spaceStr . "<ledger name=\"$self->{ledger}->{name}\" code=\"$self->{ledger}->{code}\">\n";
    $result .= $self->{ledger}->printXML(fullXML => 0, indent => ($indent + 2));  # this should only output the <ledgerInfo> tag.
    $result .= $spaceStr . "  <subEntities>\n";
    foreach my $subEntity (@{$self->{subEntities}})
    {
      $result .= $subEntity->printXML(fullXML => 0, indent => ($indent + 4));
    }
    $result .= $spaceStr . "  </subEntities>\n";
    $result .= $spaceStr . "  <journals>\n";
    foreach my $journal (@{$self->{journals}})
    {
      $result .= $journal->printXML(fullXML => 0, indent => ($indent + 4));
    }
    $result .= $spaceStr . "  </journals>\n";
    $result .= $spaceStr . "  <journalEntries>\n";
    foreach my $journalEntry (@{$self->{journalEntries}})
    {
      $result .= $journalEntry->printXML(fullXML => 0, indent => ($indent + 4));
    }
    $result .= $spaceStr . "  </journalEntries>\n";
    $result .= $spaceStr . "</ledger>\n";
  }
  else
  {
    $result .= "Data not valid!\n\n";
    $result .= join("\n", @{$valid[1]}) . "\n";
    die $result;
  }

  return $result;
}

=item string printHTML()

  This routine returns a string of html that represents this object.

=cut

sub printHTML
{
  my $self = shift;
  my $result = "";

  return $result;
}

=item string printForm( prefix => )

  This routine returns xhtml form elements that can be used to edit this object.
  A prefix can be used to uniquely identify this form from others of the same
  type.  The names of form elements can be obtained in a hash:

  use Portal::Accounting::Objects::Base;
  my %elements = %Portal::Accounting::Objects::LedgerContainer::Elements;

  !!Make a copy of the hash as shown above--NOT a refence.
  You would generally place your input from the user within the Elements hash.
  Then you can pass the Elements hash to the new method
  of a LedgerContainer object instead of individual arguments.

  foreach my $prefix (%prefixes)
    foreach $index (key %Elements)
      $tempHash{$index} = $input->{$prefix . $index}
    my object = Portal::Accounting::Objects::LedgerContainer->new(%tempHash);

=cut

sub printForm
{
  my $self = shift;
  my $result = "";

  return $result;
}

=back

=cut

1;
__END__

=head1 EXPORTED VARIABLES

 ledger         - Ledger Object
 subEntities    - array of SubEntityObject's.
 journals       - array of Journal Object's.
 journalEntries - array of TransactionObject Object's.

=head1 NOTE

 All data fields are accessible by specifying the object
 and pointing to the data member to be modified on the
 left-hand side of the assignment.
 Ex.  $obj->variable($newValue); or $value = $obj->variable;

=head1 AUTHOR

Xperience, Inc. (mailto:xiwa@pcxperience.com)

=head1 SEE ALSO

perl(1), Portal::Accounting::Objects::Base(3)

=cut
