# MonitorObject.pm - The Object Class that provides a Account Monitoring Object
# Created by James A. Pattie, 06/13/2002.
#This package is released under the GNU General Public License.
#You should have recieved a copy of the GPL with it.
# Copyright (c) 2000-2002 HLR http://www.pcxperience.org  All rights reserved.

package Portal::Accounting::Objects::MonitorObject;
use strict;
use Portal::Accounting::Objects::Base;
use Portal::XML::ConfigParser;
use Portal::XML::ConfigObject;
use vars qw($AUTOLOAD $VERSION @ISA @EXPORT);

require Exporter;

@ISA = qw(Portal::Accounting::Objects::Base Exporter AutoLoader);
@EXPORT = qw();

$VERSION = '0.02';

my %trueFalse = ( 1 => "true", 0 => "false" );
my %falseTrue = ( "true" => 1, "false" => 0 );

=head1 NAME

MonitorObject - Object used to build a Account Monitoring Object Class.

=head1 SYNOPSIS

  use Portal::Accounting::Objects::MonitorObject;
  my $monitorObj = Portal::Accounting::Objects::MonitorObject->new(
                   id => "-1", ledgerCode => "t1", accounts => "1001,2002",
                   emails => "alert@test.com", users => "admin",
                   lessThan => "0.00", greaterThan => "", operator => "and",
                   mailType => "html");
  if ($monitorObj->didErrorOccur())
  {
    die $monitorObj->errorMessage();
  }

  # now you would pass the monitorObj object to the createMonitor
  # method in the Portal::Accounting::DB::Main module.

  # This object could also have been created by a getMonitorInfo
  # or similar routine from the Portal::Accounting::DB::Main module.
  # In that case, id != "-1".

=head1 DESCRIPTION

MonitorObject is a Account Monitoring class.  This represents account(s)
to be monitored by the system (via a cron job) for activity that fits the
specified conditions.  If a match is found then the specified email
addresses and users will be email notified.

=head1 Exported FUNCTIONS

B<NOTE>: I<bool> = 1(true), 0(false)

=over 4

=item scalar new(id, xml, ledgerCode, accounts, emails, users, lessThan,
  greaterThan, mailType, operator)

 Creates a new instance of the Portal::Accounting::Objects::MonitorObject module.
 See Portal::Accounting::Objects::Base(3) for a listing of required arguments.

 id = "-1" indicates a new monitor entry.
 If id = "-1" then you must specify the ledgerCode, accounts, emails,
 users, lessThan, greaterThan, operator and mailType values, otherwise
 only xml is valid and will be processed.

=cut

sub new
{
  my $class = shift;
  my $self = $class->SUPER::new(@_);
  my %args = ( id => "-1", xml => "", ledgerCode => "", accounts => "", emails => "", users => "",
               lessThan => "", greaterThan => "", mailType => "", operator => "", @_ );

  if ($self->didErrorOccur)
  {
    $self->prefixError();
    return $self;
  }

  # instantiate anything unique to this module
  $self->{id} = $args{id};
  my $xml = $args{xml};

  eval { $self->{parserObj} = Portal::XML::ConfigParser->new(langObj => $self->{langObj}); };
  if ($@)
  {
    $self->setError("Instantiating Portal::XML::ConfigParser failed!<br>\n" . $@);
    return $self;
  }

  # now parse the xml document.
  if ($self->{id} != "-1" && (length $xml > 0))
  {
    eval { $self->{parsedObj} = $self->{parserObj}->parse(string => $xml, module => "MonitorAccounts", version => "1.0"); };
    if ($@)
    {
      $self->setError("Parsing xml = '$xml' failed!<br>\n" . $@);
      return $self;
    }

    $self->{ledgerCode} = $self->{parsedObj}->{settings}->{ledgerCode};
    $self->{accounts} = $self->{parsedObj}->{settings}->{accounts};
    $self->{emails} = $self->{parsedObj}->{settings}->{emails};
    $self->{users} = $self->{parsedObj}->{settings}->{users};
    $self->{lessThan} = $self->{parsedObj}->{settings}->{lessThan};
    $self->{greaterThan} = $self->{parsedObj}->{settings}->{greaterThan};
    $self->{mailType} = $self->{parsedObj}->{settings}->{mailType};
    $self->{operator} = $self->{parsedObj}->{settings}->{operator};
  }
  else
  {
    # make sure the ConfigObject exists and is available to the export method.
    $self->{parsedObj} = Portal::XML::ConfigObject->new(langObj => $self->{langObj});
    $self->{parsedObj}->{module} = "MonitorAccounts";
    $self->{parsedObj}->{version} = "1.0";

    $self->{ledgerCode} = $args{ledgerCode};
    $self->{accounts} = $args{accounts};
    $self->{emails} = $args{emails};
    $self->{users} = $args{users};
    $self->{lessThan} = $args{lessThan};
    $self->{greaterThan} = $args{greaterThan};
    $self->{mailType} = $args{mailType};
    $self->{operator} = $args{operator};
  }

  # do validation
  if (!$self->isValid())
  {
    # the error is set in the isValid() method.
    return $self;
  }

  # do anything else you might need to do.

  return $self;
}

=item bool isValid(void)

 Returns 0 or 1 to indicate if the object is valid.
 The error will be available via errorMessage().

=cut

sub isValid
{
  my $self = shift;

  # make sure our Parent class is valid.
  if (!$self->SUPER::isValid())
  {
    $self->prefixError();
    return 0;
  }

  # validate our parameters.
  if ($self->{id} !~ /^(-1|\d+)$/)
  {
    $self->invalid("id", $self->{id});
  }
  if ($self->{ledgerCode} !~ /^(.+)$/)
  {
    $self->invalid("ledgerCode", $self->{ledgerCode});
  }
  if ($self->{accounts} !~ /^(\d+(,\d+)*)$/)
  {
    $self->invalid("accounts", $self->{accounts});
  }
  if ($self->{emails} !~ /^(|[^@]+@[^.]+\..+(,[^@]+@[^.]+\..+)*)$/)
  {
    $self->invalid("emails", $self->{emails});
  }
  if ($self->{users} !~ /^(|\w+(,\w+)*)$/)
  {
    $self->invalid("users", $self->{users});
  }
  if ($self->{lessThan} !~ /^(|-?\d+\.\d+)$/)
  {
    $self->invalid("lessThan", $self->{lessThan});
  }
  if ($self->{greaterThan} !~ /^(|-?\d+\.\d+)$/)
  {
    $self->invalid("greaterThan", $self->{greaterThan});
  }
  if ($self->{operator} !~ /^(and|or)$/)
  {
    $self->invalid("operator", $self->{operator});
  }
  if ($self->{mailType} !~ /^(html|text)$/)
  {
    $self->invalid("mailType", $self->{mailType});
  }

  # now make sure that a valid combination is specified.
  if (length $self->{emails} == 0 && length $self->{users} == 0)
  {
    $self->missing("You must specify at least a user or an email address");
  }
  if (length $self->{lessThan} == 0 && length $self->{greaterThan} == 0)
  {
    $self->missing("You must specify a value to compare against for one of lessThan or greaterThan");
  }

  if ($self->numInvalid() > 0 || $self->numMissing() > 0)
  {
    $self->postfixError($self->genErrorString("all"));
    return 0;
  }

  return 1;
}

=item scalar export(void)

 Returns the XML document that represents this object.

=cut
sub export
{
  my $self = shift;

  $self->{parsedObj}->{settings}->{ledgerCode} = $self->{ledgerCode};
  $self->{parsedObj}->{settings}->{accounts} = $self->{accounts};
  $self->{parsedObj}->{settings}->{emails} = $self->{emails};
  $self->{parsedObj}->{settings}->{users} = $self->{users};
  $self->{parsedObj}->{settings}->{lessThan} = $self->{lessThan};
  $self->{parsedObj}->{settings}->{greaterThan} = $self->{greaterThan};
  $self->{parsedObj}->{settings}->{mailType} = $self->{mailType};
  $self->{parsedObj}->{settings}->{operator} = $self->{operator};

  my $xml;
  eval { $xml = $self->{parsedObj}->generateXML(); };
  if ($@)
  {
    $self->setError("Generation of XML failed!<br>\n" . $@);
    return $xml;
  }

  return $xml;
}

=back

=cut

1;
__END__


=head1 NOTE

 All data fields are accessible by specifying the object
 and pointing to the data member to be modified on the
 left-hand side of the assignment.
 Ex.  $obj->variable($newValue); or $value = $obj->variable;

=head1 AUTHOR

Xperience, Inc. (mailto:admin@pcxperience.com)

=head1 SEE ALSO

perl(1), Portal(3), Portal::Accounting(3), Portal::Accounting::Objects::Base(3)

=cut
