# StockObject.pm - The Object Class that provides a Stock Object
# Created by James Pattie & Jason A. Pattie, 11/07/2000.
#$Id: StockObject.pm,v 1.7 2003/09/12 21:37:55 pattieja Exp $
#This package is released under the GNU General Public License.
#You should have recieved a copy of the GPL with it.
# Copyright (c) 2000 HLR http://www.pcxperience.org  All rights reserved.

package Portal::Accounting::Objects::StockObject;
use strict;
use Portal::Accounting::Objects::Base;
use vars qw($AUTOLOAD $VERSION @ISA @EXPORT);

require Exporter;

@ISA = qw(Portal::Accounting::Objects::Base Exporter AutoLoader);
@EXPORT = qw();

$VERSION = '0.03';

=head1 NAME

StockObject - Object used to build a Stock Object Class.

=head1 SYNOPSIS

  use Portal::Accounting::Objects::StockObject;
  my $obj = Portal::Accounting::Objects::StockObject->new;
  if ($obj->didErrorOccur())
  {
    die $obj->errorMessage();
  }

=head1 DESCRIPTION

StockObject is a Stock class.

=head1 Exported FUNCTIONS

B<NOTE>: I<bool> = 1(true), 0(false)

=over 4

=item scalar new(ticker, name, number, type, market, class)

 Creates a new instance of the Portal::Accounting::Objects::StockObject module.
 See Portal::Accounting::Objects::Base(3) for a listing of required arguments.

 requires:
  ticker
  name
  number  - The SubAccount number this stock is associated with
  type
  market
  class

=cut

sub new
{
  my $class = shift;
  my $self = $class->SUPER::new(@_);
  my %args = ( ticker => "", name => "", number => 0, type => "", market => "", class => "small cap", @_ );

  if ($self->didErrorOccur)
  {
    $self->prefixError();
    return $self;
  }

  # instantiate anything unique to this module
  $self->{ticker} = $args{ticker};
  $self->{name} = $args{name};
  $self->{number} = $args{number};
  $self->{type} = $args{type};
  $self->{market} = $args{market};
  $self->{class} = $args{class};

  # do validation
  if (!$self->isValid)
  {
    # the error is set in the isValid() method.
    return $self;
  }

  # do anything else you might need to do.

  return $self;
}

=item bool isValid(void)

 Returns 0 or 1 to indicate if the object is valid.
 The error will be available via errorMessage().

=cut

sub isValid
{
  my $self = shift;

  # make sure our Parent class is valid.
  if (!$self->SUPER::isValid())
  {
    $self->prefixError();
    return 0;
  }

  # validate our parameters.
  if (length $self->{name} == 0)
  {
    $self->invalid("name", $self->{name});
  }
  if (length $self->{number} == 0)
  {
    $self->invalid("number", $self->{number});
  }
  if (length $self->{market} == 0)
  {
    $self->invalid("market", $self->{market});
  }
  if (length $self->{class} == 0)
  {
    $self->invalid("class", $self->{class});
  }
#  if (length $self->{type} == 0)
#  {
#    $self->invalid("type", $self->{type});
#  }

# It is valid for stocks not to have ticker symbols
#  if (length $self->{ticker} == 0)
#  {
#    $self->invalid("ticker", $self->{ticker});
#  }

  if ($self->numInvalid() > 0 || $self->numMissing() > 0)
  {
    $self->postfixError($self->genErrorString("all"));
    return 0;
  }

  return 1;
}

=item string printXML(fullXML bool, indent int)

 This routine returns an XML snippet for this object for use in such
 things as exporting data

 fullXML - defaults to 0 (false).  If true, we output the xml header tag.
 indent - defaults to 2.  Specifies how many spaces in our top level tag
          needs to be.  If fullXML is true, then we force 0, otherwise we
          use what the user specifies.

=cut

sub printXML
{
  my $self = shift;
  my $result = "";
  my %args = ( fullXML => 0, indent => 2, @_ );
  my $fullXML = $args{fullXML};
  my $indent = ($fullXML ? 0 : $args{indent});  # start at 0 indentation if we are making a full XML document.
  my $spaceStr = " " x $indent;

  if ($self->isValid())
  {
    if ($fullXML)
    {
      $result .= "<?xml version=\"1.0\" encoding=\"ISO-8859-1\"?>";
    }
    $result .= $spaceStr . "<stock>\n";
    foreach my $name (qw{ticker name number type market class})
    {
      if ($name eq "number")
      {
        $result .= $spaceStr . "  <code>$self->{$name}</code>\n";
      }
      else
      {
        $result .= $spaceStr . "  <$name>$self->{$name}</$name>\n";
      }
    }
    $result .= $spaceStr . "</stock>\n";
  }
  else
  {
    $result .= "Data not valid!\n\n";
    $result .= $self->errorMessage();
    die $result;
  }

  return $result;
}

=back

=cut

1;
__END__

=head1 NOTE

 All data fields are accessible by specifying the object
 and pointing to the data member to be modified on the
 left-hand side of the assignment.
 Ex.  $obj->variable($newValue); or $value = $obj->variable;

=head1 AUTHOR

Xperience, Inc. (mailto:admin@pcxperience.com)

=head1 SEE ALSO

perl(1), Portal(3), Portal::Accounting(3), Portal::Accounting::Objects::Base(3)

=cut
