# TransactionObject.pm - The Object Class that provides a Transaction Object
# that contains the referenceNum, ledgerCode, journalCode, transDate, payeePayer, memo, remember
# and an array of TransactionRowObjects.
# Created by James Pattie & Jason A. Pattie, 11/06/2000.
#$Id: TransactionObject.pm,v 1.6 2003/08/06 15:38:15 pcxuser Exp $
#This package is released under the GNU General Public License.
#You should have recieved a copy of the GPL with it.
# Copyright (c) 2000 HLR http://www.pcxperience.org  All rights reserved.

package Portal::Accounting::Objects::TransactionObject;
use strict;
use Portal::Accounting::Objects::TransactionRowObject;
use vars qw($VERSION @ISA @EXPORT $AUTOLOAD);

require Exporter;

@ISA = qw(Exporter AutoLoader);
@EXPORT = qw();

$VERSION = '0.01';

# new
# instantiates an instance of a TransactionObject
sub new
{
  my $that = shift;
  my $class = ref($that) || $that;
  my $self = bless {}, $class;
  my $errorHeader = "TransactionObject->new(): ";
  my %args = ( referenceNum => '-1', ledgerCode => undef, journalCode => 'G', transDate => undef, payeePayer => '',
               memo => '', remember => 'false', rowObjects => [], trackingId => "", origId => "", @_ );
  my ($referenceNum);

  $referenceNum = $args{'referenceNum'};

  $self->{error} = 0;  # nothing wrong yet.
  $self->{errorString} = "";

  # validate the incoming parameters
  if ($referenceNum !~ /^-?[0-9][0-9]*$/)
  {
    $self->{error} = 1;
    $self->{errorString} = $errorHeader . "Invalid reference number passed in!  The value does not appear to be numeric.";
    return $self;
  }

  $self->{referenceNum} = $referenceNum;
  $self->{ledgerCode}   = $args{'ledgerCode'};
  $self->{journalCode}  = $args{'journalCode'};
  $self->{transDate}    = $args{'transDate'};
  $self->{payeePayer}   = $args{'payeePayer'};
  $self->{memo}         = $args{'memo'};
  $self->{remember}     = $args{'remember'};
  $self->{rowObjects}   = $args{'rowObjects'};
  $self->{trackingId}   = $args{trackingId};
  $self->{origId}       = $args{origId};

  return $self;
}

sub DESTROY
{
  my $self = shift;
}

sub AUTOLOAD
{
  my $self = shift;
  my $type = ref($self) || die "$self is not an object";
  my $name = $AUTOLOAD;
  $name =~ s/.*://;	# strip fully-qualified portion
  unless (exists $self->{$name})
  {
    die "Can't access `$name' field in object of class $type";
  }
  if (@_)
  {
    return $self->{$name} = shift;
  }
  else
  {
    return $self->{$name};
  }
}

# isValid
# validates the object in its current state
sub isValid
{
  my $self = shift;
  my $errorHeader = "TransactionObject->isValid(): ";

  if ($self->{referenceNum} !~ /^-?[0-9][0-9]*/)
  {
    $self->{error} = 1;
    $self->{errorString} = $errorHeader . "The reference number does not appear to be numeric!";
    return 0;
  }
  if (not defined $self->{ledgerCode} or $self->{ledgerCode} eq "")
  {
    $self->{error} = 1;
    $self->{errorString} = $errorHeader . "The ledger code does not have a value!";
    return 0;
  }
  if (not defined $self->{journalCode} or $self->{journalCode} eq "")
  {
    $self->{error} = 1;
    $self->{errorString} = $errorHeader . "The journal code does not have a value!";
    return 0;
  }
  if (not defined $self->{transDate} or $self->{transDate} eq "")
  {
    $self->{error} = 1;
    $self->{errorString} = $errorHeader . "The transaction date does not have a value!";
    return 0;
  }
  if (not defined $self->{memo})
  {
    $self->{memo} = "";
  }
  if ((not defined $self->{remember}) or ($self->{remember} ne "true" and $self->{remember} ne "false"))
  {
    $self->{remember} = "false";
  }
  if (scalar @{ $self->{rowObjects} } < 2)
  {
    $self->{error} = 1;
    $self->{errorString} = $errorHeader . "The transaction rows array contains too few items (" . (scalar @{ $self->{rowObjects} }) . ")!";
    return 0;
  }
  my $sum = 0;
  my $debit_sum = 0;
  my $credit_sum = 0;
  for (my $rowcount=0; $rowcount < scalar @{ $self->{rowObjects} }; $rowcount++)
  {
    if (ref($self->{rowObjects}[$rowcount]) ne "Portal::Accounting::Objects::TransactionRowObject")
    {
      $self->{error} = 1;
      $self->{errorString} = $errorHeader . "Row $rowcount is not a valid TransactionRowObject (" . ref($self->{rowObjects}[$rowcount]) . ")!";
      return 0;
    }
    if (!$self->{rowObjects}[$rowcount]->isValid())
    {
      $self->{error} = 1;
      $self->{errorString} = "TransactionObject->TransactionRowObject->isValid(): Row $rowcount has invalid data! " . $self->{rowObjects}[$rowcount]->errorMessage;
      return 0;
    }
    else
    {
      if ($self->{rowObjects}[$rowcount]->dorc eq "true")
      {
        $debit_sum += $self->{rowObjects}[$rowcount]->amount;
      }
      else
      {
        $credit_sum += $self->{rowObjects}[$rowcount]->amount;
      }
    }
  }
  $sum = sprintf("%.2f", $debit_sum - $credit_sum);
  if ($sum != 0.00)
  {
    if ($debit_sum == 0.00)
    {
      $self->{error} = 1;
      $self->{errorString} = $errorHeader . "No debits entered!<br>Debit Sum: " . $debit_sum . "<br>Credit Sum: " . $credit_sum . "<br>Sum: " . $sum;
      return 0;
    }
    elsif ($credit_sum == 0.00)
    {
      $self->{error} = 1;
      $self->{errorString} = $errorHeader . "No credits entered!<br>Debit Sum: " . $debit_sum . "<br>Credit Sum: " . $credit_sum . "<br>Sum: " . $sum;
      return 0;
    }
    if ($sum < 0.00)
    {
      $self->{error} = 1;
      $self->{errorString} = $errorHeader . "Credits are greater than debits!<br>Debit Sum: " . $debit_sum . "<br>Credit Sum: " . $credit_sum . "<br>Sum: " . $sum;
      return 0;
    }
    elsif ($sum > 0.00)
    {
      $self->{error} = 1;
      $self->{errorString} = $errorHeader . "Debits are greater than credits!<br>Debit Sum: " . $debit_sum . "<br>Credit Sum: " . $credit_sum . "<br>Sum: " . $sum;
      return 0;
    }
  }

  return 1;
}

# addRow
# takes the values for a TransactionRowObject and creates a new object to add to the array of objects in the
# TransactionObject
# returns newly added row number on success, 0 on failure
sub addRow
{
  my $self = shift;
  my $errorHeader = "TransactionObject->addRow(): ";
  my %args = ( @_ );

  my $rowDate       = $args{'rowDate'};
  my $comment       = $args{'comment'};
  my $subEntity     = $args{'subEntity'};
  my $accountNum    = $args{'accountNum'};
  my $subAccountNum = $args{'subAccountNum'};
  my $dorc          = $args{'dorc'};
  my $amount        = $args{'amount'};
  my $cleared       = $args{'cleared'};

  if (not exists $args{'rowDate'})
  {
    $rowDate = $self->{transDate};
  }
  if (not exists $args{'comment'})
  {
    $comment = "";
  }
  if (not exists $args{'subEntity'})
  {
    $subEntity = "";
  }
  if (not exists $args{'accountNum'})
  {
    $self->{error} = 1;
    $self->{errorString} = "Transaction->addRow(): accountNum value missing!";
    return 0;
  }
  if (not exists $args{'subAccountNum'})
  {
    $subAccountNum = -1;
  }
  if (not exists $args{'dorc'})
  {
    $self->{error} = 1;
    $self->{errorString} = "Transaction->addRow(): dorc value missing!";
    return 0;
  }
  if (not exists $args{'amount'})
  {
    $self->{error} = 1;
    $self->{errorString} = "Transaction->addRow(): amount value missing!";
    return 0;
  }
  if (not exists $args{'cleared'})
  {
    $cleared = ' ';
  }

  # create a new TransactionRowObject with the values passed in
  my $transrowobj = Portal::Accounting::Objects::TransactionRowObject->new(rowDate => "$rowDate", comment => "$comment", subEntity => "$subEntity", accountNum => $accountNum, subAccountNum => $subAccountNum, dorc => "$dorc", amount => "$amount", cleared => "$cleared");
  if ($transrowobj->didErrorOccur())
  {
    $self->{error} = 1;
    $self->{errorString} = $errorHeader . "Failed to add row!<br>\n" . $transrowobj->errorMessage;
    return 0;
  }
  # add the row object to the current array of row objects
  $self->{rowObjects}[scalar @{$self->{rowObjects}}] = $transrowobj;

  return (scalar @{$self->{rowObjects}} - 1);
}

# deleteRow
# Removes a row from the rowObjects array
# returns 1 on success, 0 on failure
sub deleteRow
{
  my $self = shift;
  my $errorHeader = "TransactionObject->deleteRow(): ";
  my %args = ( @_ );
  my $row = $args{'row'};

  if (not exists $args{'row'})
  {
    $self->{error} = 1;
    $self->{errorString} = $errorHeader . "Row number value argument missing!";
    return 0;
  }
  if ($row !~ /^-?[0-9]+$/)
  {
    $self->{error} = 1;
    $self->{errorString} = $errorHeader . "Invalid row number argument!  The value does not appear to be numeric.";
    return 0;
  }

  if ($row > 0 && $row < scalar @{$self->{rowObjects}})
  {
    # remove the row from the array using splice to "cut" the element out
    splice(@{$self->{rowObjects}}, $row, 1);
  }

  return 1;
}

# setError
# requires: errorString
# returns: nothing
sub setError
{
  my $self = shift;
  my %args = ( errorString => "", @_ );
  my $errorString = $args{errorString};
  
  $self->{errorString} = $errorString;
  $self->{error} = 1;
}

# didErrorOccur
# returns the value of error.
sub didErrorOccur
{
  my $self = shift;

  return $self->{error};
}

# errorMessage
# returns the value of errorString
sub errorMessage
{
  my $self = shift;

  return $self->{errorString};
}

=item string printXML(fullXML bool, indent int)

 This routine returns an XML snippet for this object for use in such
 things as exporting data

 fullXML - defaults to 0 (false).  If true, we output the xml header tag.
 indent - defaults to 2.  Specifies how many spaces in our top level tag
          needs to be.  If fullXML is true, then we force 0, otherwise we
          use what the user specifies.

=cut

sub printXML
{
  my $self = shift;
  my $result = "";
  my %args = ( fullXML => 0, indent => 2, @_ );
  my $fullXML = $args{fullXML};
  my $indent = ($fullXML ? 0 : $args{indent});  # start at 0 indentation if we are making a full XML document.
  my $spaceStr = " " x $indent;

  my %mapping = ("id" => "referenceNum", "remembered" => "remember", "ledgerCode" => "ledgerCode",
                 "memo" => "memo", "journalCode" => "journalCode", "date" => "transDate",
                 "payeePayer" => "payeePayer", "origId" => "origId");

  if ($self->isValid())
  {
    if ($fullXML)
    {
      $result .= "<?xml version=\"1.0\" encoding=\"ISO-8859-1\"?>";
    }
    $result .= $spaceStr . "<journalEntry>\n";
    my @tags = qw{id remembered memo journalCode date payeePayer origId};
    push @tags, "ledgerCode" if ($fullXML);
    foreach my $name (@tags)
    {
      $result .= $spaceStr . "  <$name>" . $self->{$mapping{$name}} . "</$name>\n";
    }
    # now output the rowObjects
    $result .= $spaceStr . "  <lines>\n";
    foreach my $row (@{$self->{rowObjects}})
    {
      $result .= $row->printXML(fullXML => 0, indent => $indent + 4);
    }
    $result .= $spaceStr . "  </lines>\n";
    $result .= $spaceStr . "</journalEntry>\n";
  }
  else
  {
    $result .= "Data not valid!\n\n";
    $result .= $self->errorMessage();
    die $result;
  }

  return $result;
}

1;
__END__

=head1 NAME

TransactionObject - Object used to build a complete transaction for storage in the accounting database.

=head1 SYNOPSIS

  use TransactionObject;
  my $transobj = TransactionObject->new(referenceNum => '379',
  					ledgerCode => 'ts1',
  					journalCode => 'G',
  					transDate => '12-14-1999',
  					payeePayer => 'Bank',
  					memo => 'cashed check',
  					remember => 'false',
  					rowObjects => \@RowObjectArray,
            origId => "");
  if ($transobj->didErrorOccur())
  {
    die $transobj->errorMessage();
  }

  # set object data by specifying the data field desired
  $transobj->journalCode("I");
  $transobj->memo("");

  # make sure the object is useful by calling the validation routine
  # before writing it to the database
  if ($transobj->isValid())
  {
    updateTransaction($transobj);
  }
  else
  {
    die $transobj->errorMessage();
  }

=head1 DESCRIPTION

TransactionObject is a class that stores data for an accounting
transaction.  It provides a container for a transaction that will be used
to write to the database or update an existing record in the database.

=head1 Exported FUNCTIONS

  scalar new(referenceNum, ledgerCode, journalCode, transDate,
             payeePayer, memo, remember, rowObjects, trackingId, origId)
    Creates a new instance of the Portal::Accounting::Objects::TransactionObject
    object.  A new instance can be created in an invalid state (i.e., missing
    fields, etc.).  isValid() must be called on the object before using the
    object in a database transaction.

    trackingId = the genSysId value from an XML Transaction Import. Can be "".
    origId = the original referenceNum value for this entry.  Can be "".

  void setError(errorString)
    Sets error = 1 and $self->{errorString} = errorString.

  scalar didErrorOccur(void)
    Returns the value of error.

  scalar errorMessage(void)
    Returns the value of errorString.

  scalar isValid(void)
    Runs through the data fields contained in the class to make sure
    they contain valid data.  This function should be called before
    using this object in a database transaction.

  scalar addRow(rowDate, comment, subEntity, accountNum, subAccountNum,
                dorc, amount, cleared)
    Adds a row to the rowObjects array in the TransactionObject.
    Returns the row number of the newly added row or 0 on failure.

  scalar deleteRow(row)
    Removes the row specified from the rowObjects array.

  NOTE:  All data fields are accessible by specifying the object
         and pointing to the data member to be modified on the
         left-hand side of the assignment.

=head1 AUTHOR

James A. Pattie (mailto:james@pcxperience.com), Jason A. Pattie (mailto:pattieja@pcxperience.com)

=head1 SEE ALSO

perl(1), Transaction(3), DBIWrapper(3)

=cut
